unit e_game;

interface

uses OpenGL, windows, e_models, e_textures, e_types, e_global, e_utils, e_screen,
     e_particles, e_sound, e_records, e_about, inifiles, e_console, e_font;

type
  TTrench = class
    pos         : TPoint;
    size        : TPoint;
    angle       : single;
    point_start : TPoint;
    point_end   : TPoint;
    texture_phase : single;
    color       : TColor;
    parent      : integer;
  public
    constructor Create(_parent : integer; _point_start : TPoint; _width, _angle : single; _color : TColor);
    procedure   Render;
    procedure   Update(count : single);
    procedure   ChangeLength(_point_end : TPoint);
  end;

  TMachine = class
    id            : integer;
    pos           : TPoint;
    pos_delta     : TPoint;
    target        : TPoint;
    target_length : single;
    speed         : single;
    size          : TPoint;
    angle         : single;
    angle_target  : single;
    timer_trench  : single;
    timer_boost   : single;
    timer_quake   : single;
    trench        : integer;
    trench_color  : TColor;
    frame         : integer;
    turned        : boolean;
    fall          : integer;
    dead          : boolean;
    ghost         : boolean;
    dust          : integer;
  public
    constructor Create(_id : integer; x, _speed : single);
    procedure   Render;
    procedure   Update(count : single);
  end;

  TBonus = class
    mode        : byte;
    pos         : TPoint;
    size        : single;
    angle       : single;
    dead        : boolean;
    color       : TColor;
  public
    constructor Create;
    procedure   Render;
    procedure   Update(count : single);
  end;

  TStone = class
    mode        : byte;
    pos         : TPoint;
    size        : single;
    angle       : single;
    dead        : boolean;
    color       : TColor;
  public
    constructor Create(scale : single = 1; x : single = 1000);
    procedure   Render;
    procedure   Update(count : single);
  end;

  TGame = class
    trenches      : array of TTrench;
    stones        : array of TStone;
    machines      : array of TMachine;
    bonuses       : array of TBonus;
    machine_drag  : integer;
    machines_speed : single;

    level         : integer;
    level_timer   : single;

    score_1       : integer;
    score_2       : integer;
    score_3       : integer;
    score_timer   : single;
    score_bonus   : integer;

    frame_timer   : single;
    frame_switch  : boolean;

    ground_change : boolean;
    ground_top    : single;
    ground_color1 : TColor;
    ground_color2 : TColor;

    map_speed     : single;
    map_depth     : single;
    map_depth_old : single;
    map_depth_new : single;
    map_stones    : single;
    game_over     : boolean;
    started       : boolean;
  public
    constructor Create(r : single);
    procedure   NextLevel;
    procedure   GetScore(i : integer);
    procedure   GetBonus(mode : integer; pos : TPoint; color : TColor);
    procedure   MoveUp(var value : single; count : single);
    procedure   Explosion(pos : TPoint);
    procedure   Render;
    procedure   Update(count : single);
    procedure   Save;
    procedure   GameOver;

    function    AddTrench(_parent : integer; _point_start : TPoint; _width, _angle : single; _color : TColor) : integer;
    function    AddMachine : integer;
    function    AddBonus : integer;
    function    AddStone(scale : single = 1; x : single = 1000) : integer;
  end;

var
  game : TGame;

const
  game_height = 90;
  
implementation

uses e_main;

var
  active_trenches : integer;
  ground_colors : array[0..4] of TColor = ((r : 171; g : 119; b : 13; a : 1),
                                           (r : 187; g : 153; b : 19; a : 1),
                                           (r : 154; g : 111; b : 82; a : 1),
                                           (r : 152; g : 141; b : 78; a : 1),
                                           (r : 185; g : 113; b : 32; a : 1));

constructor TTrench.Create;
begin
parent      := _parent;
point_start := _point_start;
point_end   := point_start;
pos         := point_start;
size        := GetPoint(0, _width - 1);
angle       := _angle;
texture_phase := random;
color       := _color;
end;

procedure TTrench.Render;
begin
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glRotatef(angle, 0, 0, 1);
with size do glScalef(x + 1, y, 1);
with color do glColor4f(r * 0.9, g * 0.9, b * 0.9, 1);
SetTexture(texture_trench);

TexturePushMatrix;
TextureTranslate(texture_phase, 1);
TextureScale(size.x / 10 / 4, 1);
glCallList(model_quad);
TexturePopMatrix;

glPopMatrix;
end;

procedure TTrench.Update(count : single);
begin
game.MoveUp(point_start.y, count);
game.MoveUp(point_end.y, count);
game.MoveUp(pos.y, count);
end;

procedure TTrench.ChangeLength(_point_end : TPoint);
begin
point_end := _point_end;
pos.x     := (point_start.x + point_end.x) / 2;
pos.y     := (point_start.y + point_end.y) / 2;
size.x    := sqrt(sqr(point_start.x - point_end.x) + sqr(point_start.y - point_end.y));
end;

constructor TMachine.Create(_id : integer; x, _speed : single);
begin
id           := _id;
pos          := GetPoint(x, game_height);
pos_delta    := GetPoint(0, 0);
timer_boost  := 10000;
timer_quake  := random * 200;
target       := GetPoint(0, 1.5);
speed        := _speed;
size         := GetPoint(7, 7);
angle        := 270;
angle_target := angle;
trench       := -1;
trench_color := GetColor(171 / 255, 119 / 255, 13 / 255, 1);
frame        := random(texture_machine_frames) + 1;
fall         := -1;
dead         := false;
ghost        := true;
dust         := particles.AddEmitter(particles_dust, pos, 30, GetColor(1, 1, 1, 1));
end;

procedure TMachine.Render;
begin
glPushMatrix;
with pos do glTranslatef(x + target.x / 2, y + target.y / 2, 0);
glRotatef(angle_target, 0, 0, 1);
glScalef(target_length, 5, 1);
glColor4f(1, 1, 1, 1);
SetTexture(texture_line);
TexturePushMatrix;
TextureScale(target_length / 5, 1);
glCallList(model_quad);
TexturePopMatrix;
glPopMatrix;

glPushMatrix;
with pos do glTranslatef(x, y, 0);
with target do glTranslatef(x, y, 0);
glScalef(5, 5, 1);
glRotatef(angle_target, 0, 0, 1);
glColor4f(1, 1, 1, 1);
SetTexture(texture_arrow);
glCallList(model_quad);
glPopMatrix;    

glPushMatrix;
with pos do glTranslatef(x, y, 0);
with size do glScalef(x * 1.5, y * 1.5, 1);
glRotatef(angle, 0, 0, 1);
with pos_delta do glTranslatef(x, y, 0);
glColor4f(1, 1, 1, 1);
SetTexture(texture_machine[frame]);
glCallList(model_quad);
glPopMatrix;

if ghost then
  begin
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glPushMatrix;
    with pos do glTranslatef(x, y, 0);
    with size do glScalef(x * 1.5 + 1, y * 1.5 + 1, 1);
    glRotatef(angle, 0, 0, 1);
    with pos_delta do glTranslatef(x, y, 0);
    glColor4f(1, 1, 1, sqr(timer_boost / 5000));
    SetTexture(texture_machine[frame]);
    glCallList(model_quad);
    glPopMatrix;
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  end;
end;

function Distance(pos : TPoint; p1, p2 : TPoint) : single;
var
p0, v, w : TPoint;
c1, c2, b, d : single;
begin
v.x := p2.x - p1.x;
v.y := p2.y - p1.y;

w.x := pos.x - p1.x;
w.y := pos.y - p1.y;

c1 := w.x * v.x + w.y * v.y;
if c1 <= 0 then
  begin
    result := sqrt(sqr(pos.x - p1.x) + sqr(pos.y - p1.y));
    exit;
  end;

c2 := sqr(v.x) + sqr(v.y);
if c2 <= c1 then
  begin
    result := sqrt(sqr(pos.x - p2.x) + sqr(pos.y - p2.y));
    exit;
  end;

b := c1 / c2;
p0.x := p1.x + b * v.x;
p0.y := p1.y + b * v.y;

d := (p1.y - p2.y) * pos.x + (p2.x - p1.x) * pos.y + (p1.x * p2.y - p2.x * p1.y);
d := abs(d) / sqrt(sqr(p1.x - p2.x) + sqr(p1.y - p2.y));
result := d;
end;

procedure TMachine.Update(count : single);
var
boost_affect : single;
change_color : boolean;
i, a : integer;
begin
ghost := (timer_boost > 0);

if timer_boost > 0 then
  timer_boost := timer_boost - count;

boost_affect := 1;
if timer_boost < 2000 then
  boost_affect := timer_boost / 2000;
boost_affect := boost_affect + 1;

if fall >= 0 then
  if game.trenches[fall] <> nil then
    begin
      angle := game.trenches[fall].angle;
      speed := speed + count * 0.00001;
    end;

pos.x := pos.x + cos(angle / 180 * pi) * count * speed * boost_affect;
pos.y := pos.y + sin(angle / 180 * pi) * count * speed * boost_affect;
game.MoveUp(pos.y, count);

if pos.x < -110 then dead := true;
if pos.x >  110 then dead := true;
if pos.y < -110 then dead := true;

particles.objects[dust].pos := pos;
particles.objects[dust].color := trench_color;
TParticlesDust(particles.objects[dust]).angle := angle;

change_color := false;
if pos.y < game.ground_top + 15 then
  begin
    change_color := not SqrAspire(trench_color.r, game.ground_color2.r, count * 0.0003);
    change_color := change_color or (not SqrAspire(trench_color.g, game.ground_color2.g, count * 0.0003));
    change_color := change_color or (not SqrAspire(trench_color.b, game.ground_color2.b, count * 0.0003));
  end;

if timer_quake > 0 then
  timer_quake := timer_quake - count
else
  begin
    timer_quake := 250;
    pos_delta.x := 0;
    pos_delta.y := 0;
  end;

if game.frame_switch then
  begin
    inc(frame);
    if frame > texture_machine_frames then
      frame := 1;
  end;

//if fall < 0 then
if Aspire(angle, angle_target, count * 0.02) and (not change_color) then
  begin
    if trench >= 0 then
      if game.trenches[trench] <> nil then
        if abs(angle - game.trenches[trench].angle) > 1 then
          turned := false;
    if not turned then
      trench := game.AddTrench(id, pos, size.x, angle, trench_color);
    turned := true;
  end
else
  begin
    turned := false;
    timer_trench := timer_trench - count;
    if timer_trench < 0 then
      begin
        timer_trench := 300;
        trench := game.AddTrench(id, pos, size.x, angle, trench_color);
      end;
  end;

if trench >= 0 then
  if game.trenches[trench] <> nil then
    game.trenches[trench].ChangeLength(pos);

a := 0;
if fall >= 0 then a := fall + 1;
if a > high(game.trenches) then
  a := high(game.trenches);

if ghost then exit;

for i := high(game.trenches) downto a do
  if game.trenches[i] <> nil then
    if game.trenches[i].parent <> id then
      if Distance(pos, game.trenches[i].point_start, game.trenches[i].point_end) < 1 then
        begin
          fall := i;
          trench := -1;
        end;

if id < high(game.machines) then
  for i := id + 1 to high(game.machines) do
    if game.machines[i] <> nil then
      if not game.machines[i].ghost then
      if sqr(pos.x - game.machines[i].pos.x) + sqr(pos.y - game.machines[i].pos.y) < sqr(size.x) then
        begin
          dead := true;
          game.machines[i].dead := true;
          game.Explosion(pos);
          game.Explosion(game.machines[i].pos);
        end;
end;

constructor TBonus.Create;
begin
mode    := 1;
if random(5) = 0 then
  mode := 2;
  
pos     := GetPoint(GetRandom * 90, -game_height); 
size    := 8;
angle   := random * 360;
dead    := false;
color   := GetColor(0.5, 0.5, 0.5, 1);
case random(3) of
  0 : color.r := 1;
  1 : color.g := 1;
  2 : color.b := 1;
end;
case random(3) of
  0 : color.r := 1;
  1 : color.g := 1;
  2 : color.b := 1;
end;
end;

procedure TBonus.Render;
begin
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size + 2, size + 2, 1);
glRotatef(angle, 0, 0, 1);
glColor4f(1, 1, 1, 1);
SetTexture(texture_bonus_back);
glCallList(model_quad);
glPopMatrix;

glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size, size, 1);
SetColor(color);
case mode of
  1 : SetTexture(texture_bonus_score1);
  2 : SetTexture(texture_bonus_score2);
end;          
glCallList(model_quad);
glPopMatrix;

glBlendFunc(GL_SRC_ALPHA, GL_ONE);
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size + 1.5, size + 1.5, 1);
glColor4f(1, 1, 1, 0.3 * sqr(sqr(sin(angle / 180 * pi))));
case mode of
  1 : SetTexture(texture_bonus_score1);
  2 : SetTexture(texture_bonus_score2);
end;          
glCallList(model_quad);
glPopMatrix;
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
end;

procedure TBonus.Update(count : single);
var
i : integer;
begin
game.MoveUp(pos.y, count);

angle := angle + count * 0.04;
if angle > 360 then
  angle := 360 - angle;

if pos.y > game_height then
  begin
    dead := true;
    exit;
  end;

for i := 0 to high(game.machines) do
  if game.machines[i] <> nil then
    if not game.machines[i].ghost then
    if sqr(pos.x - game.machines[i].pos.x) + sqr(pos.y - game.machines[i].pos.y) < sqr(size - 2) then
      begin
        game.GetBonus(mode, pos, color);
        dead := true;
        exit;
      end;           
end;

constructor TStone.Create(scale : single = 1; x : single = 1000);
var
s : single;
i, m : integer;
b : boolean;
begin
mode    := random(2) + 1;
size    := (15 + random * 12) * scale;

m := 0;
repeat
  inc(m);
  if x > 900 then
    pos     := GetPoint(GetRandom * 95, -game_height - size)
  else
    pos     := GetPoint(x + GetRandom * 25, -game_height - 10 + GetRandom * 18);

  b := true;
  for i := 0 to high(game.stones) do
    if game.stones[i] <> nil then
      if game.stones[i] <> self then
        if sqr(game.stones[i].pos.x - pos.x) + sqr(game.stones[i].pos.y - pos.y) < sqr(game.stones[i].size + size) then
          begin
            b := false;
            break;
          end;

until (m > 50) or (b);

angle   := random * 360;
dead    := false;
s := 0.2 + game.ground_color2.r * 0.5 + random * 0.05;
color   := GetColor(s, s, s, 1);
end;

procedure TStone.Render;
begin
glPushMatrix;
with pos do glTranslatef(x, y, 0);
glScalef(size * 1.3, size * 1.3, 1);
glRotatef(angle, 0, 0, 1);
SetColor(color);
case mode of
  1 : SetTexture(texture_stone1);
  2 : SetTexture(texture_stone2);
end;                           
glCallList(model_quad);
glPopMatrix;
end;

procedure TStone.Update(count : single);
var
i : integer;
begin
game.MoveUp(pos.y, count);

if pos.y > game_height then
  begin
    dead := true;
    exit;
  end;

for i := 0 to high(game.machines) do
  if game.machines[i] <> nil then
    if not game.machines[i].ghost then
    if sqr(pos.x - game.machines[i].pos.x) + sqr(pos.y - game.machines[i].pos.y) < sqr(size - game.machines[i].size.x) then
      begin
        game.machines[i].dead := true;
        game.Explosion(game.machines[i].pos);
      end;
end;

constructor TGame.Create(r : single);
begin
machine_drag := -1;
machines_speed := 0.006;

map_depth     := 0;
map_depth_old := map_depth;
map_depth_new := 500;
map_stones   := 1.1;

level        := 0;
level_timer  := 1500;

score_1      := 0;
score_2      := 0;
score_3      := 0;
score_timer  := 0;
score_bonus  := 0;

frame_timer   := 0;
frame_switch  := false;

ground_change    := false;
ground_top       := -game_size;
ground_color1    := GetColor(ground_colors[0].r / 255, ground_colors[0].g / 255, ground_colors[0].b / 255, 1);
ground_color2    := ground_color1;

game_over        := false;
started          := false;
end;

procedure TGame.NextLevel;
var
s : single;
i : integer;
t : string;
begin
level_timer := 12000 + (level + 1) * 5000;

if level >= 15 then
  exit;
  
inc(level);

s := -1;
case level of
  1  : AddMachine;
  2  : map_stones := 0.996;
  3  : AddMachine;
  4  : AddMachine;
  5  : s := 0.012;
  6  : map_stones := 0.994;
  7  : AddMachine;
  8  : s := 0.015;
  9  : map_stones := 0.992;
  11 : AddMachine;
  12 : s := 0.020;
end;

t := 'th';
case level of
  1 : t := 'st';
  2 : t := 'nd';
  3 : t := 'rd';
end;
particles.AddText(GetPoint(0, -game_height), inttostr(level) + t + ' level');

if s > 0 then
  begin
    machines_speed := s;
    for i := 0 to high(machines) do
      if machines[i] <> nil then
        machines[i].speed := s;
  end;  
end;

procedure TGame.GetScore(i : integer);
begin
score_1 := score_1 + i;
if score_1 >= 1000 then
  begin
    score_1 := score_1 - 1000;
    score_2 := score_2 + 1;
    if score_2 >= 1000 then
      begin
        score_2 := score_2 - 1000;
        score_3 := score_3 + 1;
      end;
  end;
end;

procedure TGame.GetBonus(mode : integer; pos : TPoint; color : TColor);
var
score : integer;
begin
SoundPlay(sound_bonus);
case mode of
  1 : begin
        score := 100 + 10 * score_bonus;
        GetScore(score);
        score_bonus := score_bonus + 1;
        particles.AddText(pos, inttostr(score));
      end;
  2 : begin                        
        score := 200 + 20 * score_bonus;
        GetScore(score);
        score_bonus := score_bonus + 1;
        particles.AddText(pos, inttostr(score));
      end;
end;
particles.AddEmitter(particles_bonus, pos, 20, color);
end;

procedure TGame.MoveUp(var value : single; count : single);
begin
value := value + count * map_speed;
end;

procedure TGame.Explosion(pos : TPoint);
begin
particles.AddEmitter(particles_explosion, pos, 20, GetColor(1, 0.5, 0, 1));
SoundPlay(sound_explosion);
end;

procedure TGame.Render;

  function StringScore(a : integer; spaces : boolean) : string;
  var
  i1, i2 : integer;
  begin
    if a <= 0 then
      begin
        result := '   ';
        exit;
      end;

    result := inttostr(a);
    i2 := 3 - length(result);
    for i1 := 1 to i2 do
      if spaces then
        result := '0' + result
      else
        result := ' ' + result;
  end;

  function StringScores(a1, a2, a3 : integer) : string;
  begin
    result := StringScore(a1, true) + ' ' + StringScore(a2, a1 > 0) + ' ' + StringScore(a3, a2 > 0);
  end;

var
i : integer;
begin
glDisable(GL_DEPTH_TEST);

glPushMatrix;
glTranslatef(0, ground_top + game_size, 0);
glScalef((game_size * 2 + 5), game_size * 2, 1);
SetColor(ground_color1);
SetTexture(texture_map); 
TexturePushMatrix;
TextureScale(20, 20);
glCallList(model_quad);
TexturePopMatrix;

glPopMatrix;

if ground_change then
  begin
    glPushMatrix;
    glTranslatef(0, ground_top, 0);
    glScalef((game_size * 2 + 5), (game_size * 2 + 0), 1);
    SetColor(ground_color2);
    SetTexture(texture_map);
    TexturePushMatrix;
    TextureScale(20, 2);
    glCallList(model_ground);
    TexturePopMatrix;
    glPopMatrix;
  end;

glPushMatrix;
glTranslatef(0, ground_top - game_size, 0);
glScalef((game_size * 2 + 5), game_size * 2, 1);
SetColor(ground_color2);
SetTexture(texture_map);
TexturePushMatrix;
TextureScale(20, 20);
glCallList(model_quad);
TexturePopMatrix;
glPopMatrix;


for i := 0 to high(trenches) do
  if trenches[i] <> nil then
    trenches[i].Render;

for i := 0 to high(stones) do
  if stones[i] <> nil then
    stones[i].Render;

for i := 0 to high(bonuses) do
  if bonuses[i] <> nil then
    bonuses[i].Render;

particles.RenderBack(false);

for i := 0 to high(machines) do
  if machines[i] <> nil then
    machines[i].Render; 

particles.RenderFront;

if level = 0 then
  begin
    SetTexture(texture_deeper);
    glColor4f(1, 1, 1, level_timer / 1000);
    glPushMatrix;
    glScalef(405 / 3, 118 / 3, 1);
    glCallList(model_quad);
    glPopMatrix;

    glColor4f(1, 1, 1, level_timer / 1000 * 0.5);
    font.TextOut(40, 10, 16, 'little machines still goes', true);
    font.TextOut(20, 0, -70, 'press any key', true);
  end;


SetTexture(texture_cursor);
glColor4f(1, 1, 1, 1);
glPushMatrix;
with mouse_game do glTranslatef(x, y, 0);
glScalef(5, 5, 1);
glCallList(model_quad);
glPopMatrix;

font.TextOut(40, 50, 65, StringScores(score_3, score_2, score_1));

glColor4f(1, 1, 1, 0.5);
if level = 0 then
  glColor4f(1, 1, 1, 0.5 - level_timer / 1000);
font.TextOut(25, -97, -71, 'fps: ' + inttostr(fps));
font.TextOut(35, -91, 72, 'oueo', true);
font.TextOut(25, -91, 67, 'games', true);
end;

procedure TGame.Update(count : single);
var
i, j : integer;
s : single;
p : TPoint;
begin
if (not menu_active) and (not game_over) then
  game_played := true;


MoveUp(ground_top, count);
if ground_top > game_size then
  begin
    ground_top := -game_size;
    ground_color1 := ground_color2;
    ground_change := (random(4) = 0);
    if ground_change then
      begin
        i := random(high(ground_colors) + 1);
        ground_color2 := GetColor(ground_colors[i].r / 255, ground_colors[i].g / 255, ground_colors[i].b / 255, 1);
      end;
  end;


frame_switch := false;
frame_timer := frame_timer - count;
if frame_timer < 0 then
  begin
    frame_timer  := 20;
    frame_switch := true;
  end;

active_trenches := 0;
for i := 0 to high(trenches) do
  if trenches[i] <> nil then
    if trenches[i].point_end.y < game_height then
      begin
        trenches[i].Update(count);
        inc(active_trenches);
      end
    else
      begin
        trenches[i].Free;
        trenches[i] := nil;
      end;

if not mouse_down then
  machine_drag := -1;

s := 0;
j := 0;
for i := 0 to high(machines) do
  if machines[i] <> nil then
    with machines[i] do
      if not dead then
        begin
          Update(count);
          if fall < 0 then
            begin
              s := s + sin(angle / 180 * pi) * machines[i].speed;
              inc(j);
            end;

          if mouse_down and (machine_drag < 0) then
            if sqr(mouse_game.x - pos.x) + sqr(mouse_game.y - pos.y) < sqr(size.y) then
              machine_drag := i
            else
              if sqr(mouse_game.x - pos.x - target.x) + sqr(mouse_game.y - pos.y - target.y) < 5 then
                machine_drag := i;
        end
      else
        begin
          particles.objects[dust].loop := false;
          Free;
          machines[i] := nil;
        end;

if level >= 1 then
  if j = 0 then
    GameOver;

map_speed := 0.01;
if j > 0 then map_speed := -s / j;

if not game_over then
  begin
    map_depth_old := map_depth;
    map_depth := map_depth + abs(count * map_speed);

    if (map_depth > map_depth_new) and (map_depth_old < map_depth_new) then
      begin
        particles.AddText(GetPoint(0, -game_height), inttostr(round(map_depth_new)) + ' metres!');
        map_depth_new := map_depth_new + 500;
      end;

    score_timer := score_timer - count;
    if score_timer < 0 then
      begin
        score_timer := 100;
        game.GetScore(round(j * map_speed * 1000 / 5));
      end;

    if started then
      begin
        level_timer := level_timer - count;
        if level_timer < 0 then
          NextLevel;
      end;
  end;

  
if machine_drag >= 0 then
  if machines[machine_drag] <> nil then
    with machines[machine_drag] do
      begin
        angle_target := PointsAngle(mouse_game, pos);
        if angle_target < 0 then angle_target := angle_target + 360;

        if (angle_target < 200) and (angle_target >= 90) then angle_target := 200;
        if (angle_target > 340) or (angle_target < 90) then angle_target := 340;

        target_length := sqrt(sqr(mouse_game.x - pos.x) + sqr(mouse_game.y - pos.y));
        target := GetPoint(target_length * cos(angle_target / 180 * pi),
                           target_length * sin(angle_target / 180 * pi));
      end;


if level >= 1 then
  if random > 0.994 then
    AddBonus;

for i := 0 to high(bonuses) do
  if bonuses[i] <> nil then
    if not bonuses[i].dead then
      bonuses[i].Update(count)
    else
      begin
        bonuses[i].Free;
        bonuses[i] := nil;
      end;


if level >= 1 then
if random > map_stones then
  case random(5) of
    //0 : AddStone(2.5);
    1 : begin
          j := random(15) + 10;
          s := GetRandom * 95;
          for i := 0 to j do
            AddStone(0.1, s);
        end;
    else AddStone;
  end;

for i := 0 to high(stones) do
  if stones[i] <> nil then
    if not stones[i].dead then
      stones[i].Update(count)
    else
      begin
        stones[i].Free;
        stones[i] := nil;
      end;

particles.Update(count);
end;

procedure TGame.Save;
begin
end;

procedure TGame.GameOver;
begin
if game_over then exit;

game_over := true;
game_played := false;

particles.AddText(GetPoint(0, -game_height),       'game over');

particles.AddText(GetPoint(0, -game_height - 150),  '          oueo factory     ', false);
particles.AddText(GetPoint(0, -game_height - 160),  '      Grouzdev Dmitry      ', false);
particles.AddText(GetPoint(0, -game_height - 170),  '        Hramov Nickolay    ', false);

particles.AddText(GetPoint(0, -game_height - 190),  '               for igdc-39 ', false);
particles.AddText(GetPoint(0, -game_height - 200),  '               2008        ', false);

Save;
end;

function TGame.AddTrench(_parent : integer; _point_start : TPoint; _width, _angle : single; _color : TColor) : integer;
var
i, max : integer;

begin
result := -1;

if game_over then exit;

i   := 0;
max := high(trenches);
while (i <= max) and (trenches[i] <> nil) do inc(i);
if i > max then SetLength(trenches, i + 1);

if trenches[i] <> nil then trenches[i].Free;

trenches[i] := TTrench.Create(_parent, _point_start, _width, _angle, _color);

result := i;
end;

function TGame.AddMachine : integer;
var
i : integer;
x : single;
begin
result := -1;

if game_over then exit;

SetLength(machines, high(machines) + 2);
i := high(machines);

x := GetRandom * 80;
case i of
  0 : x :=   0;
  1 : x :=  35;
  2 : x := -35;
  3 : x := -70;
  4 : x :=  70;
end;
machines[i] := TMachine.Create(i, x, machines_speed);

result := i;
end;

function TGame.AddBonus : integer;
var
i, max : integer;

begin
result := -1;

if game_over then exit;

i   := 0;
max := high(bonuses);
while (i <= max) and (bonuses[i] <> nil) do inc(i);
if i > max then SetLength(bonuses, i + 1);

if bonuses[i] <> nil then bonuses[i].Free;

bonuses[i] := TBonus.Create;

result := i;
end;

function TGame.AddStone(scale : single = 1; x : single = 1000) : integer;
var
i, max : integer;

begin
result := -1;

i   := 0;
max := high(stones);
while (i <= max) and (stones[i] <> nil) do inc(i);
if i > max then SetLength(stones, i + 1);

if stones[i] <> nil then stones[i].Free;

stones[i] := TStone.Create(scale, x);

result := i;
end;

end.
