unit e_particles;

interface

uses OpenGL, e_types, e_utils, e_models, e_textures, e_font, e_screen, e_global;

const
  particles_text           = 1;
  particles_explosion      = 2;
  particles_dust           = 3;
  particles_bonus          = 4;

type
  TParticle = record
    pos                    : TPoint;
    speed                  : TPoint;
    angle                  : single;
    angle_speed            : single;
    life                   : single;
    start_life             : single;
    size                   : single;
  end;

  TParticlesEmitter = class
    objects : array of TParticle;
    pos     : TPoint;
    active  : boolean;
    render_to_texture : boolean;
    color   : TColor;
    texture : integer;
    front   : boolean;
    loop    : boolean;
    multiply : boolean;
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   virtual;
    procedure   ResetParticle(i : integer);                    virtual;
    procedure   UpdateParticle(i : integer; count : single);   virtual;
    procedure   Update(count : single);
    procedure   Render;                                        virtual;
  end;

  TParticlesText = class(TParticlesEmitter)
    text : string;
    center : boolean;
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   UpdateParticle(i : integer; count : single);   override;
    procedure   Render;                                        override;
  end;

  TParticlesExplosion = class(TParticlesEmitter)
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesDust = class(TParticlesEmitter)
    angle : single;
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   ResetParticle(i : integer);                    override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesBonus = class(TParticlesEmitter)
  public
    constructor Create(p : TPoint; n : integer; c : TColor);   override;
    procedure   UpdateParticle(i : integer; count : single);   override;
  end;

  TParticlesEngine = class
    objects : array of TParticlesEmitter;
  public
    constructor Create;
    function    AddEmitter(m : word; p : TPoint; n : integer; c : TColor) : integer;
    function    AddText(p : TPoint; text : string; center : boolean = true) : integer;
    procedure   DeleteEmitter(i : integer);
    procedure   Update(count : single);
    procedure   RenderBack(blood_only : boolean);
    procedure   RenderFront;
  end;

var
  particles             : TParticlesEngine;
  particles_active      : boolean = true;
  particles_in_frame    : integer = 0;
  particles_updates     : integer = 0;
  player_level          : integer = 1;

implementation

uses e_game;

{-- TParticlesEmitter --}

constructor TParticlesEmitter.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create;

pos     := p;
color   := c;
active  := true;
render_to_texture := false;
end;

procedure TParticlesEmitter.ResetParticle(i : integer);
begin
end;

procedure TParticlesEmitter.UpdateParticle(i : integer; count : single);
begin
end;

procedure TParticlesEmitter.Update(count : single);
var
i, j : integer;
begin
j := 0;
for i := 0 to high(objects) do
  with objects[i] do
    if life > 0 then
      begin
        life := life - count;
        UpdateParticle(i, count);
        inc(j);
      end
    else
      if loop then
        ResetParticle(i);
        
if (not loop) and (j <= 0) then active := false;
end;

procedure TParticlesEmitter.Render;
var
i : integer;
s : single;
begin
SetTexture(texture);

if multiply then
  glBlendFunc(GL_SRC_ALPHA, GL_ONE)
else
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
for i := 0 to high(objects) do
  with objects[i] do
    if life > 0 then
      begin
        inc(particles_in_frame);
        s := sqr(life / start_life);
        with color do glColor4f(r, g, b, s);
        glPushMatrix;
        with pos do glTranslatef(x, y, 0);
        glScalef(size, size, size);
        glRotatef(angle, 0, 0, 1);
        glCallList(model_quad);
        glPopMatrix;
      end;

if multiply then glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

end;

{-- TParticlesEmitter --}

{-- TParticlesText --}

constructor TParticlesText.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

text := 'text';
p := GetPoint(p.x, p.y);
front := false;
SetLength(objects, n);
for i := 0 to high(objects) do
  with objects[i] do
    begin
      pos.x := p.x;
      pos.y := p.y;

      size  := 30;

      start_life := 1000;
      life       := start_life;
    end;
end;

procedure TParticlesText.UpdateParticle(i : integer; count : single);
var
s : single;
begin
with objects[i] do
  begin
    if pos.y < game_height then
      life := start_life;
    
    game.MoveUp(pos.y, count);
  end;
end;

procedure TParticlesText.Render;
var
i : integer;
s : single;
begin
for i := 0 to high(objects) do
  with objects[i] do
    if life > 0 then
      begin
        inc(particles_in_frame);
        s := sqr(life / start_life);
        with color do glColor4f(r, g, b, s);
        font.TextOut(size, pos.x, pos.y, text, center, s);
      end;
end;

{-- TParticlesText --}

{-- TParticlesExplosion  --}

constructor TParticlesExplosion.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

//multiply := false;
texture := texture_explosion;
p := GetPoint(p.x, p.y);
front := true;
SetLength(objects, n);
for i := 0 to n - 1 do
  with objects[i] do
    begin
      a := random * 2 * pi;
      pos.x := p.x + cos(a) * 2;
      pos.y := p.y + sin(a) * 2;

      size  := 0;

      angle := random * 360;
      angle_speed := GetRandom * 0.9;

      speed := GetPoint(20 - i * 3, 0);

      start_life := 10000;
      life       := start_life;
    end;
end;

procedure TParticlesExplosion.UpdateParticle(i : integer; count : single);
var
s : single;
begin
with objects[i] do
  begin
    size := size + count * 0.004 * speed.x;
    speed.x := speed.x - count * 0.04;

    angle := angle + angle_speed * count;

    if (size < 0) and (speed.x < 0) then
      life := 0
    else
      life := start_life;

    game.MoveUp(pos.y, count);
  end;
end;

{-- TParticlesExplosion --}

{-- TParticlesDust  --}

constructor TParticlesDust.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
begin
inherited Create(p, n, c);

texture := texture_dust;

front := false;
loop := true;
angle := 0;
//multiply := true;
SetLength(objects, n);
for i := 0 to high(objects) do
  ResetParticle(i);
end;

procedure TParticlesDust.ResetParticle(i : integer);
var
p : TPoint;
a : single;
begin
p := pos;
a := angle / 180 * pi;
with objects[i] do
  begin
    pos.x := p.x;
    pos.y := p.y;

    size  := random;

    angle := random * 360;

    speed := GetPoint(cos(angle) * 0.2, sin(angle) * 0.2);

    start_life := 1000 + random * 2000;
    life       := start_life + 1500;


    life        := (2000 + 3000 * random) * 0.7;
    start_life  := life * 0.6;
  end;
end;

procedure TParticlesDust.UpdateParticle(i : integer; count : single);
var
s : single;
begin  
with objects[i] do
  begin
    size := size + count * 0.0025;

    pos.x := pos.x + speed.x * count * 0.001;
    pos.y := pos.y + speed.y * count * 0.001;

    game.MoveUp(pos.y, count);
  end;
end;

{-- TParticlesDust --}

{-- TParticlesBonus  --}

constructor TParticlesBonus.Create(p : TPoint; n : integer; c : TColor);
var
i : integer;
a : single;
begin
inherited Create(p, n, c);

texture := texture_debris;
p := GetPoint(p.x, p.y);
front := true;
SetLength(objects, n);
for i := 0 to high(objects) do
  with objects[i] do
    begin
      pos.x := p.x;
      pos.y := p.y;

      size  := 2 + random * 2;

      angle := random * 360;

      speed := GetPoint(cos(angle / 180 * pi) * 10, sin(angle / 180 * pi) * 10);

      start_life := 1000;
      life       := start_life;
    end;
end;

procedure TParticlesBonus.UpdateParticle(i : integer; count : single);
var
s : single;
begin
with objects[i] do
  begin
    pos.x := pos.x + speed.x * count * 0.001;
    pos.y := pos.y + speed.y * count * 0.001;

    game.MoveUp(pos.y, count);
  end;
end;

{-- TParticlesBonus --}

{-- TParticlesEngine --}

constructor TParticlesEngine.Create;
var
i : integer;
x, z : single;
begin
SetLength(objects, 0);
end;

function TParticlesEngine.AddEmitter(m : word; p : TPoint; n : integer; c : TColor) : integer;
var
i, max : integer;
begin
result := -1;

i   := 0;
max := high(objects);
while (i <= max) and (objects[i] <> nil) do inc(i);
if i > max then SetLength(objects, max + 4);

case m of
  particles_text       : objects[i] := TParticlesText.Create(p, n, c);
  particles_explosion  : objects[i] := TParticlesExplosion.Create(p, n, c);
  particles_dust       : objects[i] := TParticlesDust.Create(p, n, c);
  particles_bonus      : objects[i] := TParticlesBonus.Create(p, n, c);
end;

result := i;
end;

function TParticlesEngine.AddText(p : TPoint; text : string; center : boolean = true) : integer;
begin
result := AddEmitter(particles_text, p, 1, GetColor(1, 1, 1, 1));
TParticlesText(objects[result]).text := text;
TParticlesText(objects[result]).center := center;
end;

procedure TParticlesEngine.DeleteEmitter(i : integer);
begin
objects[i].Free;
objects[i] := nil;
end;

procedure TParticlesEngine.Update(count : single);
var
i : integer;
begin
if not particles_active then exit;

particles_updates := 0;
for i := 0 to high(objects) do
  if objects[i] <> nil then
    if objects[i].active then
      begin
        objects[i].Update(count);
        inc(particles_updates);
      end
    else
      DeleteEmitter(i);
end;

procedure TParticlesEngine.RenderBack(blood_only : boolean);
var
i : integer;
begin
particles_in_frame := 0;

if not particles_active then exit;

for i := 0 to high(objects) do
  if objects[i] <> nil then
    if objects[i].active and (not objects[i].front) then
      //if (not blood_only) or (blood_only and (objects[i] is TParticlesBlood)) then
      //if (blood_only and ((objects[i] is TParticlesBlood) or (objects[i] is TParticlesExplosionSoot))) then
      if (not blood_only) or (objects[i].render_to_texture and blood_only) then
        objects[i].Render;
end;

procedure TParticlesEngine.RenderFront;
var
i : integer;
begin
if not particles_active then exit;

for i := 0 to high(objects) do
  if objects[i] <> nil then
    if objects[i].active and objects[i].front then
      objects[i].Render;
end;

{-- TParticlesEngine --}

end.
